package com.biganto.visual.roompark.presentation.screen.deals.util

import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import butterknife.BindView
import butterknife.ButterKnife
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.domain.model.StatusModel
import com.biganto.visual.roompark.presentation.screen.deals.DealPreviewModel
import com.biganto.visual.roompark.util.extensions.toRubly
import com.biganto.visual.roompark.util.view_utils.status_progress_view.StatusProgressAnimationState
import com.biganto.visual.roompark.util.view_utils.status_progress_view.StatusProgressCeil
import com.google.android.material.textview.MaterialTextView

/**
 * Created by Vladislav Bogdashkin on 16.10.2019.
 */

class DealsListAdapter : RecyclerView.Adapter<DealViewHolder>() {

    private var list: MutableList<DealPreviewModel> = mutableListOf()

    fun addItems(list:List<DealPreviewModel>){
        this.list.clear()
        this.list.addAll(list)
        notifyDataSetChanged()
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): DealViewHolder =
        DealViewHolder(
            LayoutInflater.from(parent.context)
                .inflate(R.layout.deal_card_viewholder, parent, false)
        )

    override fun getItemCount(): Int = list.size

    override fun onBindViewHolder(holder: DealViewHolder, position: Int) {
        holder.bindModel(list[position])
    }
}


class DealViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {

    @BindView(R.id.deal_card_header) lateinit var estateTitle: TextView
    @BindView(R.id.common_info_block) lateinit var commonInfo:View
    @BindView(R.id.start_tour_button) lateinit var startTour:View

    @BindView(R.id.info_ceil_1) lateinit var info1:View
    @BindView(R.id.info_ceil_2) lateinit var info2:View
    @BindView(R.id.info_ceil_3) lateinit var info3:View
    @BindView(R.id.info_ceil_4) lateinit var info4:View

    @BindView(R.id.deal_sum_value_text_view) lateinit var dealSum:TextView
    @BindView(R.id.deal_payed_value_text_view) lateinit var dealPayed:TextView
    @BindView(R.id.deal_to_pay_value_text_view) lateinit var dealSumToPay:TextView

    @BindView(R.id.progress_holder) lateinit var statusContainer:LinearLayout

    @BindView(R.id.statusContainer) lateinit var statusLayout:LinearLayout


    private fun typeName(type:String) = when(type){
        "flat" -> "КВАРТИРА"
        "parking" -> "МАШИНОМЕСТО"
        "storage" -> "КЛАДОВКА"
        else -> "ОБЪЕКТ"
    }

    init {
        ButterKnife.bind(this, itemView)
    }

    fun bindModel(deal: DealPreviewModel) {
        val fullObjName = "${typeName(deal.type)}\n№${deal.name}"
        estateTitle.text = fullObjName
        renderCommonInfo(deal)

        if (deal.dealTourIds.isNullOrEmpty()) startTour.visibility = View.GONE
        else startTour.visibility = View.VISIBLE

        dealSum.text = deal.dealSum.toRubly()
        dealPayed.text = deal.dealPayout.toRubly()
        dealSumToPay.text = deal.dealToPay.toRubly()

        createStatusView(
            statusLayout,
            statusContainer,
            deal.statusList,
            deal.statusNo,
            false
        )
    }

    private fun renderCommonInfo(info:DealPreviewModel){

        if (info.building == null) info1.visibility = View.GONE
        else { info1.title().text = "Корпус"; info1.text().text = info.building.toString()}

        if (info.section == null) info1.visibility = View.GONE
        else { info2.title().text = "Секция"; info2.text().text = info.section.toString()}

        if (info.floor == null) info1.visibility = View.GONE
        else { info3.title().text = "Этаж"; info3.text().text = info.floor.toString()}

        if (info.area == null) info1.visibility = View.GONE
        else { info4.title().text = "Общая, м²"; info4.text().text = info.area.toString()}
    }

    private fun View.title() = this.findViewById<TextView>(R.id.info_ceil_header)
    private fun View.text() = this.findViewById<TextView>(R.id.info_ceil_content)

}

fun createStatusView(mainContainer: LinearLayout,container:ViewGroup,list:List<StatusModel>,currentStatus:Int, isVertical:Boolean){

    container.removeAllViews()
    val scale = container.resources.displayMetrics.density.toDouble()
    val param24dp = kotlin.math.ceil(24 * scale).toInt()
    val wrapContent = ViewGroup.LayoutParams.WRAP_CONTENT

    (container as LinearLayout).weightSum = list.size.toFloat()

    list.sortedBy { it.orderId }.forEach{
        val ceil = LayoutInflater.from(container.context)
            .inflate(R.layout.status_view,container,false) as StatusProgressCeil
        ceil.layoutParams.height = if (isVertical) wrapContent else param24dp
        ceil.layoutParams.width = if (isVertical) param24dp else wrapContent
        (ceil.layoutParams as LinearLayout.LayoutParams).weight = 1f
        (ceil.layoutParams as LinearLayout.LayoutParams).gravity = Gravity.BOTTOM
        ceil.setHasEnd(it.orderId != list.size)
        ceil.setHasStart(it.orderId>1)
        ceil.setIsEnabled(it.orderId <= currentStatus)
        ceil.setNext(it.orderId < currentStatus)
        if (it.orderId > currentStatus)
            ceil.setAnimState(StatusProgressAnimationState.DISABLE)
        ceil.invalidate()

        container.addView(ceil)
    }

    container.invalidate()
    val currentStatusText = LayoutInflater.from(container.context)
        .inflate(R.layout.status_title_textview,container,false) as MaterialTextView
    currentStatusText.text = list[currentStatus-1].title

    mainContainer.addView(currentStatusText,0)
    mainContainer.invalidate()





}