package com.biganto.visual.roompark.presentation.screen.feeds

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.FeedsInteractor
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class FeedsScreenPresenter @Inject constructor(
    private val interactor: FeedsInteractor
)
    : BigantoBasePresenter<FeedsScreen, FeedsScreenViewState>() {

    private val restoreModel = RestoreModel(
        arrayListOf(),
        arrayListOf(),
        arrayListOf(),
        arrayListOf())


    override fun attachView(view: FeedsScreen) {
        Timber.d("AttachView ")
        restoreStateObservable.accept(FeedsScreenViewState.RestoreView(restoreModel))
        super.attachView(view)
    }

    override fun bindIntents() {

        val fetchCams = interactor.fetchCams()
            .map {
                restoreModel.cams = it.items
                FeedsScreenViewState.CamsList(it.items)
            }

        val fetchAlbums = interactor.fetchAlbums()
            .map {
                restoreModel.albums = it
                FeedsScreenViewState.AlbumsPages(it)
            }

        val fetchFeeds = interactor.fetchTopFeeds()
            .map {
                restoreModel.feeds = it.feeds
                FeedsScreenViewState.FeedsPages(it.feeds.toList())
            }

        val  getFeedArticlesPreview = intent(FeedsScreen::feedsTabSelected)
            .flatMapSingle { interactor.fetchArticles(it) }
            .map {
                restoreModel.articles = it.articles
                FeedsScreenViewState.GetFeedArticlesPreview(it.articles.toList())
            }

        val state = restoreStateObservable
            .mergeWith(fetchFeeds)
            .mergeWith(getFeedArticlesPreview)
            .mergeWith(fetchAlbums)
            .mergeWith(fetchCams)
            .doOnError{ Timber.e(it)}
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(FeedsScreenViewState::class.java), FeedsScreen::render)
    }

}